"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const spawn = require("cross-spawn");
const moment = require("moment");
const path = require("path");
const ordering_project_deps_1 = require("../components/project_deps/ordering_project_deps");
const sync_with_linked_modules_1 = require("../components/project_deps/sync_with_linked_modules");
const prettier_1 = require("../config/prettier");
const tslint_1 = require("../config/tslint");
const typescript_1 = require("../config/typescript");
const dockerignore_1 = require("../meta/dockerignore");
const editorConfig_1 = require("../meta/editorConfig");
const gitignore_1 = require("../meta/gitignore");
const Npmrc_1 = require("../meta/Npmrc");
function build_node_server() {
    return __awaiter(this, void 0, void 0, function* () {
        // Config file
        typescript_1.typescriptConfig.init(true);
        tslint_1.tsLintConfig.init(true);
        prettier_1.prettierConfig.init(true);
        // Meta files init
        dockerignore_1.dockerIgnore.init(true);
        gitignore_1.gitIgnore.init(true);
        editorConfig_1.editorConfig.init(true);
        Npmrc_1.npmrc.init(true);
        console.log("");
        yield ordering_project_deps_1.ordering_project_deps();
        yield sync_with_linked_modules_1.sync_with_linked_modules();
        console.log(chalk_1.default.bold.blue(`[ BUILD_NODE_SERVER ][ START ]`));
        const startTime = moment();
        console.log("");
        const result = spawn.sync("tsc", [
            "--project",
            path.resolve(process.cwd(), "tsconfig.json"),
            "--rootDir",
            path.resolve(process.cwd(), ""),
            "--outDir",
            path.resolve(process.cwd(), "dist"),
            "--module",
            "commonjs",
        ], {
            cwd: process.cwd(),
            encoding: "utf8",
            stdio: "inherit",
        });
        if (result.signal) {
            if (result.signal === "SIGKILL") {
                console.log(chalk_1.default.bold.red("The tsc failed because the process exited too early. " +
                    "This probably means the system ran out of memory or someone called `kill -9` on the process."));
                process.exit(1);
            }
            else if (result.signal === "SIGTERM") {
                console.log(chalk_1.default.bold.red("The tsc failed because the process exited too early. " +
                    "Someone might have called `kill` or `killall`, or the system could be shutting down."));
                process.exit(1);
            }
        }
        console.log("");
        console.log(chalk_1.default.bold.blue(`[ BUILD_NODE_SERVER ][ END ][ ${moment().diff(startTime, "milliseconds")} ms ]`));
        console.log("");
    });
}
build_node_server();
