"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const copy = require("copy");
const del = require("del");
const fs_1 = require("fs");
const lodash_1 = require("lodash");
const mkdirp = require("mkdirp");
const moment = require("moment");
const path = require("path");
const env_1 = require("../../config/env");
const rearguard_1 = require("../../config/rearguard");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
const const_1 = require("../../const");
// tslint:disable:variable-name
function copy_bundles() {
    return __awaiter(this, void 0, void 0, function* () {
        if ((env_1.envConfig.isWDS || env_1.envConfig.isBuild) &&
            !(env_1.envConfig.has_dll || env_1.envConfig.has_ui_lib || env_1.envConfig.has_node_lib)) {
            const startTime = moment();
            console.log(chalk_1.default.bold.blue(`==============COPY_BUNDLES=============`));
            console.log("");
            /////////////////////
            //
            // START OF PROCEDURE
            //
            /////////////////////
            try {
                for (const module of rearguard_1.rearguardConfig.sync_project_deps) {
                    const module_path = env_1.envConfig.resolveLocalModule(module);
                    const { has_dll, has_ui_lib, pkg } = new RearguardConfig_1.RearguardConfig(env_1.envConfig, path.resolve(module_path, "package.json"));
                    if (has_dll || has_ui_lib) {
                        yield copy_bundle(module_path, const_1.DLL_BUNDLE_DIR_NAME, pkg.name);
                    }
                    if (has_ui_lib) {
                        yield copy_bundle(module_path, const_1.LIB_BUNDLE_DIR_NAME, pkg.name);
                    }
                }
            }
            catch (error) {
                console.error(error);
                process.exit(1);
            }
            /////////////////////
            //
            // END OF PROCEDURE
            //
            /////////////////////
            const endTime = moment();
            console.log("");
            console.log(chalk_1.default.bold.blue(`[ SPEED ][ ${endTime.diff(startTime, "milliseconds")} ms ]`));
            console.log(chalk_1.default.bold.blue(`=======================================`));
            console.log("");
        }
    });
}
exports.copy_bundles = copy_bundles;
function copy_bundle(module_path, bundle_dirname, pkg_name) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // Имя пакета (package.json).name;
            const name = lodash_1.snakeCase(pkg_name);
            // Путь до бандла из установленого модуля.
            const source_bundle = path.resolve(module_path, bundle_dirname, name);
            // Путь до бандла который находится в проекте.
            const target_bundle = path.resolve(process.cwd(), bundle_dirname, name);
            // Удаляю бандл из проекта.
            if (fs_1.existsSync(target_bundle)) {
                const paths = yield del(target_bundle);
                for (const item of paths) {
                    console.log(chalk_1.default.gray(`[ REMOVE ][ ${name} ][ ${path.relative(process.cwd(), item)} ]`));
                }
            }
            /////////////////////
            //
            // Если существует бандл в node_modules то создаю директорию и копирую в неё содержимое источника.
            //
            /////////////////////
            if (fs_1.existsSync(source_bundle)) {
                yield new Promise((resolve, reject) => {
                    mkdirp(target_bundle, (error) => {
                        if (error) {
                            reject(error);
                        }
                        else {
                            console.log(chalk_1.default.green(`[ CREATED ][ DIR ][ ${name} ][ ${path.relative(process.cwd(), target_bundle)} ]`));
                            resolve();
                        }
                    });
                });
                yield new Promise((resolve, reject) => {
                    copy([`${source_bundle}/**`], target_bundle, (error, items) => {
                        if (!error) {
                            if (env_1.envConfig.isDebug) {
                                for (const item of items) {
                                    console.log(chalk_1.default.cyan(`[ COPY ][ ${name} ][ ${path.relative(process.cwd(), item.path)} ]`));
                                }
                            }
                            else {
                                console.log(chalk_1.default.cyan(`[ COPY ][ ${name} ][ ${items.length} FILES ]`));
                            }
                            resolve();
                        }
                        else {
                            reject(error);
                        }
                    });
                });
                console.log("");
            }
        }
        catch (error) {
            console.error(error);
            process.exit(1);
        }
    });
}
// tslint:enable:variable-name
