"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const path = require("path");
const env_1 = require("../../config/env");
const rearguard_1 = require("../../config/rearguard");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
const helpers_1 = require("../../helpers");
// tslint:disable:variable-name object-literal-sort-keys
function get_bundles_info() {
    /////////////////////
    //
    // START OF PROCEDURE
    //
    /////////////////////
    const result = [];
    try {
        for (const module of rearguard_1.rearguardConfig.sync_project_deps) {
            // Абсолютный путь до директории модуля, который находится в локальном node_modules.
            const module_path = env_1.envConfig.resolveLocalModule(module);
            const config = new RearguardConfig_1.RearguardConfig(env_1.envConfig, path.resolve(module_path, "package.json"));
            const bundle_name = lodash_1.snakeCase(config.pkg.name);
            const bundleInfo = {
                has_dll: config.has_dll,
                has_ui_lib: config.has_ui_lib,
                load_on_demand: config.load_on_demand,
                // Имя модуля
                pkg_name: config.pkg.name,
                // Имя бандал в snake_case.
                bundle_name,
                // Абсолютный путь к субдиректории (deb/prod) бандла.
                // Имя точки входа, ключь для assets.
                bundle_entry_name: {
                    dll: config.has_dll ? helpers_1.dll_entry_name(bundle_name) : "",
                    lib: config.has_ui_lib ? helpers_1.lib_entry_name(bundle_name) : "",
                },
                assets: {
                    dll: config.has_dll ? helpers_1.dll_assets_path(module_path, bundle_name) : "",
                    lib: config.has_ui_lib ? helpers_1.lib_assets_path(module_path, bundle_name) : "",
                },
                manifest: config.has_dll ? helpers_1.dll_manifest_path(module_path, bundle_name) : "",
            };
            result.push(bundleInfo);
        }
    }
    catch (error) {
        console.error(error);
        process.exit(1);
    }
    return result;
    /////////////////////
    //
    // END OF PROCEDURE
    //
    /////////////////////
}
exports.get_bundles_info = get_bundles_info;
// tslint:enable:variable-name
