"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const isomorphic_1 = require("@borodindmitriy/isomorphic");
const chalk_1 = require("chalk");
const chokidar = require("chokidar");
const fs_1 = require("fs");
const path = require("path");
const BuildStatusConfig_1 = require("../config/buildStatus/BuildStatusConfig");
const env_1 = require("../config/env");
const rearguard_1 = require("../config/rearguard");
const const_1 = require("../const");
const copy_bundles_1 = require("./project_deps/copy_bundles");
const delete_bundles_1 = require("./project_deps/delete_bundles");
const ordering_project_deps_1 = require("./project_deps/ordering_project_deps");
const sync_with_linked_modules_1 = require("./project_deps/sync_with_linked_modules");
// tslint:disable:variable-name
let global_modules_watcher;
let local_modules_watcher;
let sync_project_deps;
exports.watch_deps_event_emitter = new isomorphic_1.EventEmitter();
function doSync() {
    return __awaiter(this, void 0, void 0, function* () {
        const cur_sync_project_deps = rearguard_1.rearguardConfig.sync_project_deps.join(", ");
        if (sync_project_deps !== cur_sync_project_deps) {
            return watch_deps();
        }
        yield ordering_project_deps_1.ordering_project_deps();
        yield sync_with_linked_modules_1.sync_with_linked_modules();
        if (rearguard_1.rearguardConfig.has_project || rearguard_1.rearguardConfig.has_dll || rearguard_1.rearguardConfig.has_ui_lib) {
            yield delete_bundles_1.delete_bundles();
            yield copy_bundles_1.copy_bundles();
        }
    });
}
function watch_deps() {
    if (global_modules_watcher) {
        global_modules_watcher.close();
    }
    if (local_modules_watcher) {
        local_modules_watcher.close();
    }
    sync_project_deps = rearguard_1.rearguardConfig.sync_project_deps.join(", ");
    const global_modules = [];
    const local_modules = [];
    /////////////////////
    //
    // Проверка на существование модулей;
    // Составление списка модулей;
    //
    /////////////////////
    for (const name of rearguard_1.rearguardConfig.sync_project_deps) {
        const global_path = env_1.envConfig.resolveGlobalModule(name);
        const local_path = env_1.envConfig.resolveLocalModule(name);
        if (fs_1.existsSync(global_path)) {
            console.log(chalk_1.default.white(`[ WATCH ][ GLOBAL_MODULE: ${global_path} ]`));
            global_modules.push(path.resolve(global_path, const_1.NON_VERSIONABLE_CONFIG_FILE_NAME));
        }
        else if (fs_1.existsSync(local_path)) {
            console.log(chalk_1.default.bold.yellow(`[ WATCH ][ LOCAL_MODULE: ${local_path} ]`));
            local_modules.push(path.resolve(local_path, const_1.LIB_DIR_NAME, "**/*"));
            local_modules.push(path.resolve(local_path, const_1.DLL_BUNDLE_DIR_NAME, "**/*"));
            local_modules.push(path.resolve(local_path, const_1.LIB_BUNDLE_DIR_NAME, "**/*"));
        }
        else {
            console.log(chalk_1.default.red(`[ ERROR ]` +
                `[ You haven't link in global node_modules ${global_path} or local node_modules ${local_path} ]`));
            console.log(chalk_1.default.red(`[ ERROR ]` + `[ You need install ${name} module or link to global node_modules ]`));
            process.exit(1);
        }
    }
    console.log("");
    // END
    const options = {
        cwd: process.cwd(),
        followSymlinks: false,
        ignoreInitial: true,
    };
    local_modules_watcher = chokidar.watch(local_modules, options);
    local_modules_watcher.on("all", (type, watched_file) => __awaiter(this, void 0, void 0, function* () {
        yield doSync();
        exports.watch_deps_event_emitter.emit("SYNCED");
    }));
    global_modules_watcher = chokidar.watch(global_modules, options);
    global_modules_watcher.on("all", (type, watched_file) => __awaiter(this, void 0, void 0, function* () {
        const { status } = new BuildStatusConfig_1.BuildStatusConfig(watched_file);
        if (status === "done") {
            yield doSync();
            exports.watch_deps_event_emitter.emit("SYNCED");
        }
    }));
}
exports.watch_deps = watch_deps;
// tslint:enable:variable-name
